@@ -1,10 +1,13 @@
 package Tk::NBFrame; 
 require Tk;
-require DynaLoader;
 
-@ISA = qw(DynaLoader Tk::Widget);
 
-Tk::Widget->Construct('NBFrame');
+use vars qw($VERSION);
+$VERSION = '3.003'; # $Id: //depot/Tk8/NBFrame/NBFrame.pm#3$
+
+@ISA = qw(Tk::Widget);
+
+Construct Tk::Widget 'NBFrame';
 
 bootstrap Tk::NBFrame $Tk::VERSION; 
 
