@@ -1,9 +1,13 @@
 package Tk::ColorEditor;
 
+
+use vars qw($VERSION);
+$VERSION = '3.003'; # $Id: //depot/Tk8/Tk/ColorEditor.pm#3$
+
 use Tk qw(lsearch Ev);
 use Tk::Toplevel;
 @Tk::ColorEditor::ISA = qw(Tk::Toplevel);
-Tk::Widget->Construct('ColorEditor');
+Construct Tk::Widget 'ColorEditor';
 
 %Tk::ColorEditor::names = ();
 
@@ -131,7 +135,6 @@ BEGIN {
     $SET_PALETTE = 'Set Palette';
 }
 
-
 use subs qw(color_space hsvToRgb rgbToHsv);
 
 # ColorEditor public methods.
@@ -450,7 +453,6 @@ sub Populate
     $middle_middle[1]->pack(-side => 'top', -expand => 1);
     $middle_middle[2]->pack(-side => 'top', -expand => 1);
     $middle_middle[3]->pack(-side => 'top', -expand => 1, -fill => 'x');
-    my (@label, @scale);
     $cw->{'Labels'} = ["zero","one","two"];
     foreach $i (0..2) {
         $label[$i] = $cw->Label(-textvariable => \$cw->{'Labels'}[$i]);
@@ -478,8 +480,9 @@ sub Populate
         -borderwidth  => 2,
         -textvariable => \$cw->{'Entry'},
         -width        => 10,
-        -font         => "-Adobe-Courier-Medium-R-Normal-*-120-*-*-*-*-*-*",
-    );
+        -font         => "-*-Courier-Medium-R-Normal--*-120-*-*-*-*-*-*"
+    );                   
+
     $nameLabel->pack(-in => $middle_middle[3], -side => 'left');
     $name->pack(
         -in     => $middle_middle[3],
@@ -507,8 +510,9 @@ sub Populate
     my $value = $cw->Label(
         -textvariable => \$cw->{'color'}, 
         -width        => 13,
-        -font         => '-Adobe-Courier-Medium-R-Normal-*-120-*-*-*-*-*-*',
-    );
+        -font         => "-*-Courier-Medium-R-Normal--*-120-*-*-*-*-*-*"
+    );                   
+
     $swatch->pack(
         -in     => $middle_right,
         -side   => 'top', 
@@ -563,7 +567,7 @@ sub Populate
                                $middle->cget('-background')],
         -command        => ['CALLBACK', undef, undef, ['set_colors',$cw]],
         '-highlight'    => ['METHOD', undef, undef, 'background'],
-        -cursor         => ['DESCENDANTS', 'cursor', 'Cursor', 'top_left_ptr'],
+        -cursor         => ['DESCENDANTS', 'cursor', 'Cursor', 'left_ptr'],
     );
 
 } # end Populate, ColorEditor constructor
@@ -600,8 +604,8 @@ sub set_colors {
             );
             $objref->update;
         } 
-        eval {$color = ($widget->configure("-\L${type}"))[3]} if $reset;
-        eval {$widget->configure("-\L${type}" => $color)};
+        eval {local $SIG{'__DIE__'}; $color = ($widget->configure("-\L${type}"))[3]} if $reset;
+        eval {local $SIG{'__DIE__'}; $widget->configure("-\L${type}" => $color)};
     }
 
     $objref->{'Status'}->withdraw if $display;
@@ -691,10 +695,11 @@ sub color
 	 return;
        }
        ($red,$green,$blue) = $name =~ /$format/;
+       # Looks like a call for 'pack' or similar rather than eval
        eval "\$red = 0x$red; \$green = 0x$green; \$blue = 0x$blue;";
-       $red = $red << 										$shift;
+       $red   = $red   << $shift;
        $green = $green << $shift;
-       $blue = $blue << $shift;
+       $blue  = $blue  << $shift;
     }
    $objref->{'red'} = $red;
    $objref->{'blue'} = $blue;
