@@ -9,11 +9,23 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkWinWindow.c 1.13 96/04/11 17:52:20
+ * SCCS: @(#) tkWinWindow.c 1.23 97/07/01 18:14:13
  */
 
 #include "tkWinInt.h"
 
+/*
+ * The windowTable maps from HWND to Tk_Window handles.
+ */
+
+static Tcl_HashTable windowTable;
+
+/*
+ * Have statics in this module been initialized?
+ */
+
+static int initialized = 0;
+
 /*
  * Forward declarations for procedures defined in this file:
  */
@@ -26,7 +38,189 @@ static void 		StackWindow _ANSI_ARGS_((Window w, Window sibling,
 /*
  *----------------------------------------------------------------------
  *
- * TkMakeWindow --
+ * Tk_AttachHWND --
+ *
+ *	This function binds an HWND and a reflection procedure to
+ *	the specified Tk_Window. 
+ *
+ * Results:
+ *	Returns an X Window that encapsulates the HWND.
+ *
+ * Side effects:
+ *	May allocate a new X Window.  Also enters the HWND into the
+ *	global window table.
+ *
+ *----------------------------------------------------------------------
+ */
+
+Window
+Tk_AttachHWND(tkwin, hwnd)
+    Tk_Window tkwin;
+    HWND hwnd;
+{
+    int new;
+    Tcl_HashEntry *entryPtr;
+    TkWinDrawable *twdPtr = (TkWinDrawable *) Tk_WindowId(tkwin);
+
+    if (!initialized) {
+	Tcl_InitHashTable(&windowTable, TCL_ONE_WORD_KEYS);
+	initialized = 1;
+    }
+
+    /*
+     * Allocate a new drawable if necessary.  Otherwise, remove the
+     * previous HWND from from the window table.
+     */
+
+    if (twdPtr == NULL) {
+	twdPtr = (TkWinDrawable*) ckalloc(sizeof(TkWinDrawable));
+	twdPtr->type = TWD_WINDOW;
+	twdPtr->window.winPtr = (TkWindow *) tkwin;
+    } else if (twdPtr->window.handle != NULL) {
+	entryPtr = Tcl_FindHashEntry(&windowTable,
+		(char *)twdPtr->window.handle);
+	Tcl_DeleteHashEntry(entryPtr);
+    }
+
+    /*
+     * Insert the new HWND into the window table.
+     */
+
+    twdPtr->window.handle = hwnd;
+    entryPtr = Tcl_CreateHashEntry(&windowTable, (char *)hwnd, &new);
+    Tcl_SetHashValue(entryPtr, (ClientData)tkwin);
+
+    return (Window)twdPtr;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tk_HWNDToWindow --
+ *
+ *	This function retrieves a Tk_Window from the window table
+ *	given an HWND.
+ *
+ * Results:
+ *	Returns the matching Tk_Window.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+Tk_Window
+Tk_HWNDToWindow(hwnd)
+    HWND hwnd;
+{
+    Tcl_HashEntry *entryPtr = Tcl_FindHashEntry(&windowTable, (char*)hwnd);
+    if (entryPtr != NULL) {
+	return (Tk_Window) Tcl_GetHashValue(entryPtr);
+    }
+    return NULL;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tk_GetHWND --
+ *
+ *	This function extracts the HWND from an X Window.
+ *
+ * Results:
+ *	Returns the HWND associated with the Window.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+HWND
+Tk_GetHWND(window)
+    Window window;
+{
+    TkWinDrawable *twdPtr = (TkWinDrawable *) window;
+    return twdPtr->window.handle;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkpPrintWindowId --
+ *
+ *	This routine stores the string representation of the
+ *	platform dependent window handle for an X Window in the
+ *	given buffer.
+ *
+ * Results:
+ *	Returns the result in the specified buffer.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TkpPrintWindowId(buf, window)
+    char *buf;			/* Pointer to string large enough to hold
+				 * the hex representation of a pointer. */
+    Window window;		/* Window to be printed into buffer. */
+{
+    HWND hwnd = (window) ? Tk_GetHWND(window) : 0;
+    sprintf(buf, "0x%x", (unsigned int) hwnd);
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkpScanWindowId --
+ *
+ *	Given a string which represents the platform dependent window
+ *	handle, produce the X Window id for the window.
+ *
+ * Results:
+ *	The return value is normally TCL_OK;  in this case *idPtr
+ *	will be set to the X Window id equivalent to string.  If
+ *	string is improperly formed then TCL_ERROR is returned and
+ *	an error message will be left in interp->result.  If the
+ *	number does not correspond to a Tk Window, then *idPtr will
+ *	be set to None.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+TkpScanWindowId(interp, string, idPtr)
+    Tcl_Interp *interp;		/* Interpreter to use for error reporting. */
+    Arg string;			/* String containing a (possibly signed)
+				 * integer in a form acceptable to strtol. */
+    int *idPtr;			/* Place to store converted result. */
+{
+    int number;
+    Tk_Window tkwin;
+
+    if (Tcl_GetInt(interp, string, &number) != TCL_OK) {
+	return TCL_ERROR;
+    }
+    tkwin = Tk_HWNDToWindow((HWND)number);
+    if (tkwin) {
+	*idPtr = Tk_WindowId(tkwin);
+    } else {
+	*idPtr = None;
+    }
+    return TCL_OK;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkpMakeWindow --
  *
  *	Creates a Windows window object based on the current attributes
  *	of the specified TkWindow.
@@ -41,25 +235,16 @@ static void 		StackWindow _ANSI_ARGS_((Window w, Window sibling,
  */
 
 Window
-TkMakeWindow(winPtr, parent)
+TkpMakeWindow(winPtr, parent)
     TkWindow *winPtr;
     Window parent;
 {
     HWND parentWin;
-    TkWinDrawable *twdPtr;
     int style;
+    HWND hwnd;
     
-
-    twdPtr = (TkWinDrawable*) ckalloc(sizeof(TkWinDrawable));
-    if (twdPtr == NULL) {
-	return None;
-    }
-
-    twdPtr->type = TWD_WINDOW;
-    twdPtr->window.winPtr = winPtr;
-
     if (parent != None) {
-	parentWin = TkWinGetHWND(parent);
+	parentWin = Tk_GetHWND(parent);
 	style = WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS;
     } else {
 #ifdef __OPEN32__
@@ -80,17 +265,17 @@ TkMakeWindow(winPtr, parent)
 #endif
     }
 
-    twdPtr->window.handle = CreateWindow(TK_WIN_CHILD_CLASS_NAME, "", style,
-	    winPtr->changes.x, winPtr->changes.y,
-	    winPtr->changes.width, winPtr->changes.height,
-	    parentWin, NULL, TkWinGetAppInstance(), twdPtr);
+    /*
+     * Create the window, then ensure that it is at the top of the
+     * stacking order.
+     */
 
-    if (twdPtr->window.handle == NULL) {
-	ckfree((char *) twdPtr);
-	twdPtr = NULL;
-    }
-
-    return (Window)twdPtr;
+    hwnd = CreateWindow(TK_WIN_CHILD_CLASS_NAME, NULL, style,
+	    Tk_X(winPtr), Tk_Y(winPtr), Tk_Width(winPtr), Tk_Height(winPtr),
+	    parentWin, NULL, Tk_GetHINSTANCE(), NULL);
+    SetWindowPos(hwnd, HWND_TOP, 0, 0, 0, 0,
+		    SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
+    return Tk_AttachHWND((Tk_Window)winPtr, hwnd);
 }
 
 /*
@@ -115,29 +300,33 @@ XDestroyWindow(display, w)
     Display* display;
     Window w;
 {
+    Tcl_HashEntry *entryPtr;
     TkWinDrawable *twdPtr = (TkWinDrawable *)w;
     TkWindow *winPtr = TkWinGetWinPtr(w);
-    HWND hwnd = TkWinGetHWND(w);
+    HWND hwnd = Tk_GetHWND(w);
 
     display->request++;
 
     /*
-     * Remove references to the window in the pointer module, and 
-     * then remove the backpointer from the drawable.
+     * Remove references to the window in the pointer module then
+     * release the drawable.
      */
 
-    TkWinPointerDeadWindow(winPtr);
-    twdPtr->window.winPtr = NULL;
+    TkPointerDeadWindow(winPtr);
+
+    entryPtr = Tcl_FindHashEntry(&windowTable, (char*)hwnd);
+    if (entryPtr != NULL) {
+	Tcl_DeleteHashEntry(entryPtr);
+    }
+
+    ckfree((char *)twdPtr);
 
     /*
      * Don't bother destroying the window if we are going to destroy
-     * the parent later.  Also if the window has already been destroyed
-     * then we need to free the drawable now.
+     * the parent later.
      */
 
-    if (!hwnd) {
-	ckfree((char *)twdPtr);
-    } else if (!(winPtr->flags & TK_PARENT_DESTROYED)) {
+    if (hwnd != NULL && !(winPtr->flags & TK_DONT_DESTROY_WINDOW)) {
 	DestroyWindow(hwnd);
     }
 }
@@ -173,15 +362,6 @@ XMapWindow(display, w)
     ShowWindow(TkWinGetHWND(w), SW_SHOWNORMAL);
     winPtr->flags |= TK_MAPPED;
 
-    event.type = MapNotify;
-    event.xmap.serial = display->request;
-    event.xmap.send_event = False;
-    event.xmap.display = display;
-    event.xmap.event = winPtr->window;
-    event.xmap.window = winPtr->window;
-    event.xmap.override_redirect = winPtr->atts.override_redirect;
-    Tk_QueueWindowEvent(&event, TCL_QUEUE_TAIL);
-
     /*
      * Check to see if this window is visible now.  If all of the parent
      * windows up to the first toplevel are mapped, then this window and
@@ -198,14 +378,27 @@ XMapWindow(display, w)
 		break;
 	    }
 	}
+    } else {
+	event.type = MapNotify;
+	event.xmap.serial = display->request;
+	event.xmap.send_event = False;
+	event.xmap.display = display;
+	event.xmap.event = winPtr->window;
+	event.xmap.window = winPtr->window;
+	event.xmap.override_redirect = winPtr->atts.override_redirect;
+	Tk_QueueWindowEvent(&event, TCL_QUEUE_TAIL);
     }
-
+    
     /*
      * Generate VisibilityNotify events for this window and its mapped
      * children.
      */
 
     event.type = VisibilityNotify;
+    event.xvisibility.serial = display->request;
+    event.xvisibility.send_event = False;
+    event.xvisibility.display = display;
+    event.xvisibility.window = winPtr->window;
     event.xvisibility.state = VisibilityUnobscured;
     NotifyVisibility(&event, winPtr);
 }
@@ -235,9 +428,11 @@ NotifyVisibility(eventPtr, winPtr)
     XEvent *eventPtr;		/* Initialized VisibilityNotify event. */
     TkWindow *winPtr;		/* Window to notify. */
 {
-    eventPtr->xvisibility.window = winPtr->window;
-    Tk_QueueWindowEvent(eventPtr, TCL_QUEUE_TAIL);
-    for (winPtr = winPtr->childList; winPtr != NULL;
+    if (winPtr->atts.event_mask & VisibilityChangeMask) {
+	eventPtr->xvisibility.window = winPtr->window;
+	Tk_QueueWindowEvent(eventPtr, TCL_QUEUE_TAIL);
+    }
+    for (winPtr = winPtr->childList; winPtr != NULL; 
 	    winPtr = winPtr->nextPtr) {
 	if (winPtr->flags & TK_MAPPED) {
 	    NotifyVisibility(eventPtr, winPtr);
@@ -272,17 +467,24 @@ XUnmapWindow(display, w)
 
     display->request++;
 
+    /*
+     * Bug fix: Don't short circuit this routine based on TK_MAPPED because
+     * it will be cleared before XUnmapWindow is called.
+     */
+
     ShowWindow(TkWinGetHWND(w), SW_HIDE);
     winPtr->flags &= ~TK_MAPPED;
 
-    event.type = UnmapNotify;
-    event.xunmap.serial = display->request;
-    event.xunmap.send_event = False;
-    event.xunmap.display = display;
-    event.xunmap.event = winPtr->window;
-    event.xunmap.window = winPtr->window;
-    event.xunmap.from_configure = False;
-    Tk_QueueWindowEvent(&event, TCL_QUEUE_TAIL);
+    if (winPtr->flags & TK_TOP_LEVEL) {
+	event.type = UnmapNotify;
+	event.xunmap.serial = display->request;
+	event.xunmap.send_event = False;
+	event.xunmap.display = display;
+	event.xunmap.event = winPtr->window;
+	event.xunmap.window = winPtr->window;
+	event.xunmap.from_configure = False;
+	Tk_HandleEvent(&event);
+    }
 }
 
 /*
@@ -458,8 +660,7 @@ XConfigureWindow(display, w, value_mask, values)
     XWindowChanges* values;
 {
     TkWindow *winPtr = TkWinGetWinPtr(w);
-    HWND window = TkWinGetHWND(w);
-    HWND insertAfter;
+    HWND hwnd = TkWinGetHWND(w);
 
     display->request++;
 
@@ -468,7 +669,7 @@ XConfigureWindow(display, w, value_mask, values)
      */
 
     if (value_mask & (CWX|CWY|CWWidth|CWHeight)) {
-	MoveWindow(window, winPtr->changes.x, winPtr->changes.y,
+	MoveWindow(hwnd, winPtr->changes.x, winPtr->changes.y,
 		winPtr->changes.width, winPtr->changes.height, TRUE);
     }
 
@@ -477,30 +678,14 @@ XConfigureWindow(display, w, value_mask, values)
      */
 
     if (value_mask & CWStackMode) {
+	HWND sibling;
 	if ((value_mask & CWSibling) && (values->sibling != None)) {
-	    HWND sibling = TkWinGetHWND(values->sibling);
-
-	    /*
-	     * Windows doesn't support the Above mode, so we insert the
-	     * window just below the sibling and then swap them.
-	     */
-
-	    if (values->stack_mode == Above) {
-		SetWindowPos(window, sibling, 0, 0, 0, 0,
-			SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
-		insertAfter = window;
-		window = sibling;
-	    } else {
-		insertAfter = sibling;
-	    }
+	    sibling = Tk_GetHWND(values->sibling);
 	} else {
-	    insertAfter = (values->stack_mode == Above) ? HWND_TOP
-		: HWND_BOTTOM;
+	    sibling = NULL;
 	}
-		
-	SetWindowPos(window, insertAfter, 0, 0, 0, 0,
-		SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
-    } 
+	TkWinSetWindowPos(hwnd, sibling, values->stack_mode);
+    }
 }
 
 /*
@@ -578,3 +763,75 @@ XChangeWindowAttributes(display, w, valueMask, attributes)
 	XDefineCursor(display, w, attributes->cursor);
     }
 }
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkWinSetWindowPos --
+ *
+ *	Adjust the stacking order of a window relative to a second
+ *	window (or NULL).
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Moves the specified window in the stacking order.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TkWinSetWindowPos(hwnd, siblingHwnd, pos)
+    HWND hwnd;			/* Window to restack. */
+    HWND siblingHwnd;		/* Sibling window. */
+    int pos;			/* One of Above or Below. */
+{
+    HWND temp;
+
+    /*
+     * Since Windows does not support Above mode, we place the
+     * specified window below the sibling and then swap them.
+     */
+
+    if (siblingHwnd) {
+	if (pos == Above) {
+	    SetWindowPos(hwnd, siblingHwnd, 0, 0, 0, 0,
+		    SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
+	    temp = hwnd;
+	    hwnd = siblingHwnd;
+	    siblingHwnd = temp;
+	}
+    } else {
+	siblingHwnd = (pos == Above) ? HWND_TOP : HWND_BOTTOM;
+    }
+
+    SetWindowPos(hwnd, siblingHwnd, 0, 0, 0, 0,
+	    SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkpWindowWasRecentlyDeleted --
+ *
+ *	Determines whether we know if the window given as argument was
+ *	recently deleted. Called by the generic code error handler to
+ *	handle BadWindow events.
+ *
+ * Results:
+ *	Always 0. We do not keep this information on Windows.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+TkpWindowWasRecentlyDeleted(win, dispPtr)
+    Window win;
+    TkDisplay *dispPtr;
+{
+    return 0;
+}
